﻿using System;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;
using System.Collections.ObjectModel;

namespace Post7670Sample
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            winformsMap1.MapUnit = GeographyUnit.DecimalDegree;
            winformsMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.GeographicColors.ShallowOcean);

            string shapeFilePath = @"C:\Program Files\ThinkGeo\Map Suite Desktop Evaluation Edition 3.0\Samples\SampleData\Data\Countries02.shp";
            ShapeFileFeatureLayer worldLayer = new ShapeFileFeatureLayer(shapeFilePath);
            worldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.SimpleColors.Transparent, GeoColor.FromArgb(100, GeoColor.SimpleColors.Black));

            InMemoryFeatureLayer highlightLayer = new InMemoryFeatureLayer();
            highlightLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            highlightLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(100, GeoColor.StandardColors.DarkGreen));

            InMemoryFeatureLayer gridLayer = new InMemoryFeatureLayer();
            gridLayer.Open();
            gridLayer.Columns.Add(new FeatureSourceColumn("PageNumber", "string", 20));
            gridLayer.Close();

            gridLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            gridLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(50, GeoColor.StandardColors.Red), GeoColor.SimpleColors.Black);
            gridLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = TextStyles.City1("PageNumber");

            LayerOverlay worldOverlay = new LayerOverlay();
            worldOverlay.Layers.Add("WorldLayer", worldLayer);
            winformsMap1.Overlays.Add("WorldOverlay", worldOverlay);

            LayerOverlay highlightOverlay = new LayerOverlay();
            highlightOverlay.Layers.Add("HighlightLayer", highlightLayer);
            highlightOverlay.Layers.Add("GridLayer", gridLayer);
            winformsMap1.Overlays.Add("HighlightOverlay", highlightOverlay);

            winformsMap1.CurrentExtent = new RectangleShape(-177.333203125, 92.96484375, 82.823046875, -89.84765625);
            winformsMap1.Refresh();
        }

        private void button2_Click(object sender, EventArgs e)
        {
            FeatureLayer worldLayer = winformsMap1.FindFeatureLayer("WorldLayer");
            worldLayer.Open();
            Feature feature = worldLayer.QueryTools.GetFeatureById("47", new string[0]);
            worldLayer.Close();
            RectangleShape featureBoundingBox = feature.GetBoundingBox();

            InMemoryFeatureLayer highlightLayer = (InMemoryFeatureLayer)winformsMap1.FindFeatureLayer("HighlightLayer");
            highlightLayer.InternalFeatures.Clear();
            highlightLayer.InternalFeatures.Add("47", feature);

            InMemoryFeatureLayer gridLayer = (InMemoryFeatureLayer)winformsMap1.FindFeatureLayer("GridLayer");
            gridLayer.InternalFeatures.Clear();
            int rowCount = 10;
            int columnCount = 5;
            Collection<RectangleShape> allPageExtents = GetGridCells(featureBoundingBox, rowCount, columnCount);
            Collection<RectangleShape> allPageIntessectsExtents = GetIntersectionCells(feature.GetShape(), allPageExtents);
            int pageNumber = 0;
            foreach (RectangleShape pageExtent in allPageIntessectsExtents)
            {
                Feature pageFeature = new Feature(pageExtent);
                pageFeature.ColumnValues.Add("PageNumber", string.Format("Page:{0}", pageNumber.ToString()));
                pageNumber++;
                gridLayer.InternalFeatures.Add(pageFeature);
            }
            winformsMap1.CurrentExtent = featureBoundingBox;
            winformsMap1.Refresh(winformsMap1.Overlays["HighlightOverlay"]);
        }

        private Collection<RectangleShape> GetGridCells(RectangleShape boundingBox, int rowCount, int columnCount)
        {
            double upperLefX = boundingBox.UpperLeftPoint.X;
            double upperLeftY = boundingBox.UpperLeftPoint.Y;
            double width = boundingBox.Width / columnCount;
            double height = boundingBox.Height / rowCount;

            Collection<RectangleShape> returnCells = new Collection<RectangleShape>();
            for (int i = 0; i < rowCount; i++)
            {
                for (int j = 0; j < columnCount; j++)
                {
                    double x1 = upperLefX + j * width;
                    double y1 = upperLeftY - i * height;

                    double x2 = x1 + width;
                    double y2 = y1 - height;

                    RectangleShape returnCell = new RectangleShape(x1, y1, x2, y2);
                    returnCells.Add(returnCell);
                }
            }
            return returnCells;
        }

        private Collection<RectangleShape> GetIntersectionCells(BaseShape baseShape, Collection<RectangleShape> gridCells)
        {
            Collection<RectangleShape> returnCells = new Collection<RectangleShape>();
            foreach (RectangleShape gridCell in gridCells)
            {
                if (baseShape.Intersects(gridCell))
                {
                    returnCells.Add(gridCell);
                }
            }
            return returnCells;

        }
    }
}
